/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertySheetPage;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry;
import eu.etaxonomy.taxeditor.propertysheet.PropertySourceAdapter;

/**
 * The abstract editor for displaying a category of <code>Taxon</code> data, corresponding
 * to the tabs ("Name", "Descriptive", etc.) at the bottom of a <code>Taxon</code> view. Implements
 * <code>IAdaptable</code> in order to display properties of the objects whose UI elements have focus. 
 * <p>
 * Implementing classes can choose to show an object in the property sheet when the
 * <code>AbstractTaxonEditorView</code> gets focus, by passing the object to the method
 * <code>setDefaultPropertySheetObject</code>, for instance, in the method<code>init</code>.
 * </p>
 * @author p.ciardelli
 * @created 10.09.2008
 * @version 1.0
 */
public abstract class AbstractTaxonEditorView extends EditorPart implements
		IAdaptable {
	private static final Logger logger = Logger
			.getLogger(AbstractTaxonEditorView.class);

	private Taxon taxon;
	
	/**
	 * When this <code>EditorPart</code> gets focus, the data structure of 
	 * <code>defaultPropertyObject</code> is displayed in the property sheet.
	 */
	protected Object defaultPropertySheetObject = null;
	
	protected IManagedForm managedForm;
	protected ScrolledForm scrolledForm; 
	protected Composite parent;
	protected ISelectionProvider provider;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
				"Invalid Input: Must be IFileEditorInput");
		
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			throw new PartInitException(
				"Invalid Input: Taxon cannot be null");
		}

		setSite(site);
		setInput(input);
		
		this.provider = new SimpleSelectionProvider();
		this.getSite().setSelectionProvider(provider);
	}

	/**
	 * When this <code>EditorPart</code> gets focus, the data structure of 
	 * <code>defaultPropertyObject</code> is displayed in the property sheet.
	 * 
	 * @param object
	 */
	protected void setDefaultPropertySheetObject(Object object) {
		this.defaultPropertySheetObject = object;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		setSelection(defaultPropertySheetObject);
	}
	
	/**
	 * If there is a default property sheet object with a corresponding property source class, 
	 * display it in the property sheet. Otherwise, empty the property sheet with an empty
	 * <code>StructuredSelection</code>.
	 * 
	 * @param selectedObject
	 */
	protected void setSelection(Object selectedObject) {
		IPropertySource propertySource = null;
		if (selectedObject != null) {
			propertySource = new PropertySourceAdapter(selectedObject).getPropertySource();
		}
		
		if (propertySource == null) {
			provider.setSelection(new StructuredSelection());
		} else {
			provider.setSelection(new StructuredSelection(propertySource));
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite composite) {
		
		managedForm = new ManagedForm(composite) {
			public void dirtyStateChanged() {
				firePropertyChange(PROP_DIRTY);
			}
			public boolean setInput(Object input) {
				setSelection(input);
				return super.setInput(input); 
			} 
		};
		scrolledForm = managedForm.getForm();
		parent = scrolledForm.getBody();
				
		Taxon taxon = getTaxon();
		parent.setData(taxon);
		
		parent.setLayout(new TableWrapLayout());		
		parent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
	}
	
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class type) {
	    if (type == IPropertySheetPage.class) {
	    	
	        PropertySheetPage page = new PropertySheetPage() {
	            public void makeContributions(IMenuManager menuManager,
	                    IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
	            	super.makeContributions(menuManager, toolBarManager, statusLineManager);
	            
	            	// Remove "Show categories", "Show advanced properties", "Restore default value"
	            	toolBarManager.removeAll();
	            	menuManager.removeAll();
	            }
	            
	            public Control getControl() {
	            	Control control = super.getControl();
	            	if (!control.isDisposed()) {
	            		if (control instanceof Tree) {
	            			UiUtil.setPropertySheetTree((Tree) control);
	            		}
	            	}
	            	return control;
	            }
	        };	        
	        UiUtil.setPropertySheetPage(page);
	        
	        CustomSortPropertySheetEntry entry = new CustomSortPropertySheetEntry();
	        page.setRootEntry(entry);
	        page.refresh();
	        
	        return page;
	    }
	    return super.getAdapter(type);
	}
	
	protected Taxon getTaxon() {
		return taxon;
	}
}
