package eu.etaxonomy.taxeditor.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.taxeditor.actions.WidgetTransfer;

/**
 * A composite which can be dragged between different EditorGroupComposite. The user
 * uses the draggable control for dragging the whole composite.
 * 
 * @see eu.etaxonomy.taxeditor.editor.EditorGroupComposite
 * 
 * @author p.ciardelli
 * 
 */
abstract public class GroupedComposite extends Composite {

	IParentDataAdapter parentDataAdapter;
	private Control draggableControl;

	public GroupedComposite(Composite parent) {
		super(parent, SWT.NONE);

		createContent();
	}

	abstract protected void createContent();

	protected void setDraggableControl(Control control) {
		draggableControl = control;
	}

	public void setDraggable(boolean draggable) {

		if (draggable) {

			if (dragger != null) {
				// Already initalized
				return;
			}

			if (draggableControl == null) {
				throw new NullPointerException(
						"Draggable control must be set to add draggability");
			}

			Transfer[] types = new Transfer[] { WidgetTransfer.getInstance() };
			int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;

			dragger = new DragSource(draggableControl, operations);
			dragger.setTransfer(types);
			dragger.addDragListener(dragSourceListener);

		} else {
			dragger = null;
		}
	}

	/**
	 * Drag listener which passes the Composite as the data in a drag event.
	 */
	DragSourceListener dragSourceListener = new DragSourceAdapter() {

		public void dragStart(DragSourceEvent event) {
			GroupedComposite.this.setFocus();
			event.doit = true;
		}

		public void dragSetData(DragSourceEvent event) {
			WidgetTransfer.getInstance().setWidget(GroupedComposite.this);
		}
	};
	private DragSource dragger;

	/**
	 * Override of setParent: dispose of parent if this is its last child
	 * composite
	 * 
	 * @see org.eclipse.swt.widgets.Control#setParent(org.eclipse.swt.widgets.Composite)
	 */
	public boolean setParent(Composite parent) {
		
		Composite oldParent = this.getParent();
		Composite oldGrandparent = oldParent.getParent();
		
		if (super.setParent(parent)) {
			if (oldParent instanceof EditorGroupComposite
					&& oldParent.getChildren().length == 0) {
				oldParent.dispose();
			}
			adaptParentData();

			// Redraw the component holding the group and grouped composites
			parent.getParent().layout();
			
			// In case we are dragging between different views, redraw the
			//	"old" view
			if (!oldGrandparent.equals(parent.getParent())) {
				oldGrandparent.layout();	
			}

			return true;
		}
		return false;
	}

	public void dispose () {
		Composite parent = this.getParent();
		Composite grandParent = null;
		if (parent != null) {
			grandParent = parent.getParent();
		}
		super.dispose();
		if (parent instanceof Composite
				&& parent.getChildren().length == 0) {
			parent.dispose();
		}
		
		// Redraw the component holding the group and grouped composites
		if (grandParent != null) {
			grandParent.layout();
		}
	}
	
	/**
	 * A grouped composite is usually grouped according to data held by its
	 * parent. When the composite changes parents, this method gets called to
	 * update data as needed.
	 */
	protected void adaptParentData() {
		if (parentDataAdapter != null) {
			parentDataAdapter.adaptParentData();
		}
	}

	public void setParentDataAdapter(IParentDataAdapter parentDataAdapter) {
		this.parentDataAdapter = parentDataAdapter;
	}
}