/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.TreeItem;

import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.navigation.RecentNamesTableViewer;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeLabelProvider;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;

/**
 * This is a generic dialog that allows the user to choose a taxon via the
 * taxonomic tree, recently selected names, or search.
 * 
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class SelectTaxonDialog extends Dialog {
	private static final Logger logger = Logger
			.getLogger(SelectTaxonDialog.class);

	public static final String TAXON_TO_SYNONYM = "taxon_to_synonym";
	public static final String TAXON_TO_NEW_PARENT = "taxon_to_new_parent";

	private String dialogType;
	private String shellText;
	private String descriptionText;
	private Text taxonNameText;

	private TaxonomicTreeViewer treeViewer;
	private Taxon taxon;
	private Taxon destinationTaxon;

	private List<Button> buttons = new ArrayList<Button>();

	private Composite selectComposite;

	public SelectTaxonDialog(Shell parentShell, String dialogType) {
		super(parentShell);
		this.dialogType = dialogType;
	}

	public Taxon open(Taxon taxon) {
		this.taxon = taxon;

		if (dialogType.equals(TAXON_TO_SYNONYM)) {
			this.shellText = "Choose an accepted name for \""
					+ CdmUtil.getDisplayName(taxon) + "\"";
			this.descriptionText = "You have chosen to turn \""
					+ CdmUtil.getDisplayName(taxon)
					+ "\" into a synonym. Please choose a new accepted name from one of the following options.";
		}

		if (dialogType.equals(TAXON_TO_NEW_PARENT)) {
			this.shellText = "Choose a new parent for \""
					+ CdmUtil.getDisplayName(taxon) + "\"";
			this.descriptionText = "You have chosen to select a new parent for \""
					+ CdmUtil.getDisplayName(taxon)
					+ "\". Please choose the new parent taxon from one of the following options.";
		}
		
		int returnCode = super.open();
		if (returnCode == OK) {
			return destinationTaxon;
		}
		return null;
	}

	/**
	 * Create contents of the dialog
	 * 
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		parent.setRedraw(true);
		Composite container = (Composite) super.createDialogArea(parent);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		container.setLayout(gridLayout);

		final Label descriptionLabel = new Label(container, SWT.WRAP);
		descriptionLabel.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, true,
				false, 2, 1));
		descriptionLabel.setText(descriptionText);

		final Group taxTreeGroup = new Group(container, SWT.NONE);
		taxTreeGroup.setLayout(new GridLayout());
		final GridData gd_group_1 = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd_group_1.heightHint = 83;
		taxTreeGroup.setLayoutData(gd_group_1);

		final Button chooseTreeButton = new Button(taxTreeGroup, SWT.RADIO);
		buttons.add(chooseTreeButton);
		chooseTreeButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false));
		chooseTreeButton.setText("Choose from taxonomic tree");

		Composite taxTreeComposite = new Composite(taxTreeGroup, SWT.BORDER);
		taxTreeComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		taxTreeComposite.setLayout(new FillLayout());
		taxTreeComposite.setBackground(Display.getCurrent().getSystemColor(
				SWT.COLOR_WHITE));

		treeViewer = new TaxonomicTreeViewer(taxTreeComposite);

		IObservableSet observableTaxonSet = TaxEditorPlugin.getDefault()
				.getObservableSessionTaxa();
		treeViewer.setLabelProvider(new TaxonomicTreeLabelProvider(
				observableTaxonSet) {

		});
		treeViewer.getTree().addFocusListener(
				new SelectButtonFocusListener(chooseTreeButton));
		treeViewer.addDoubleClickListener(new SelectTaxonDoubleClickListener());
		treeViewer.getTree().addListener(SWT.PaintItem, new Listener() {
			/** If MenuItem with taxon is being drawn, put it in italics.
			 * 
			 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
			 **/
			public void handleEvent(Event event) {
				Object data = event.item.getData();
				if (data instanceof Taxon && ((Taxon) data).equals(taxon)) {
					((TreeItem) event.item).setFont(TaxEditorPlugin.getDefault().getFont
							(ITaxEditorConstants.MENU_ITEM_ITALICS_FONT));
				}
			}
		});

		// Open up tree to reveal taxon
		treeViewer.revealTaxon(taxon);

		final Composite rightPaneComposite = new Composite(container, SWT.NONE);
		rightPaneComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true));
		rightPaneComposite.setLayout(new FillLayout(SWT.VERTICAL
				| SWT.HORIZONTAL));

		final Group recentNamesGroup = new Group(rightPaneComposite, SWT.NONE);
		recentNamesGroup.setLayout(new GridLayout());

		final Button chooseRecentNamesButton = new Button(recentNamesGroup,
				SWT.RADIO);
		buttons.add(chooseRecentNamesButton);
		chooseRecentNamesButton.setText("Choose from recently viewed taxa");

		Composite recentNamesComposite = new Composite(recentNamesGroup,
				SWT.BORDER);
		recentNamesComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL,
				true, true));
		recentNamesComposite.setLayout(new FillLayout());
		recentNamesComposite.setBackground(Display.getCurrent().getSystemColor(
				SWT.COLOR_WHITE));

		TableViewer recentNamesViewer = new RecentNamesTableViewer(
				recentNamesComposite);
		recentNamesViewer
				.addDoubleClickListener(new SelectTaxonDoubleClickListener());
		recentNamesViewer.getTable().addFocusListener(
				new SelectButtonFocusListener(chooseRecentNamesButton));

		final Group searchGroup = new Group(rightPaneComposite, SWT.NONE);
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		searchGroup.setLayout(gridLayout_1);

		final Button searchForNameButton = new Button(searchGroup, SWT.RADIO);
		buttons.add(searchForNameButton);
		searchForNameButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER,
				true, false));
		searchForNameButton.setText("Search for name of accepted taxon");
		new Label(searchGroup, SWT.NONE);

		final Text searchText = new Text(searchGroup, SWT.BORDER);
		searchText
				.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		searchText.setForeground(SWTResourceManager.getColor(192, 192, 192));
		searchText.setText("Use \"*\" for wildcard searching");
		searchText.addFocusListener(new SelectButtonFocusListener(
				searchForNameButton) {

			public void focusGained(FocusEvent e) {
				selectButton(button);
				searchText.setForeground(SWTResourceManager.getColor(0, 0, 0));
				searchText.setText("");
			}

			public void focusLost(FocusEvent e) {
				if (searchText.getText() == "") {
					searchText.setForeground(SWTResourceManager.getColor(192,
							192, 192));
					searchText.setText("Use \"*\" for wildcard searching");
				}
			}

		});

		final Button searchButton = new Button(searchGroup, SWT.NONE);
		searchButton.setLayoutData(new GridData());
		searchButton.setText("Search");

		selectComposite = new Composite(container, SWT.NONE);
		selectComposite.setLayout(new RowLayout());
		selectComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.BOTTOM, true,
				false, 2, 1));

		taxonNameText = new Text(selectComposite, SWT.BORDER);
		taxonNameText.setFont(SWTResourceManager.getFont("", 12, SWT.BOLD));
		taxonNameText.setBackground(SWTResourceManager.getColor(255, 255, 255));
		final RowData rd_text = new RowData();
		rd_text.width = 400;
		taxonNameText.setLayoutData(rd_text);
		taxonNameText.setEditable(false);

		//
		initButtons();

		return container;
	}

	public void setSelectedTaxon(Taxon taxon) {

		this.destinationTaxon = taxon;
		TaxonNameBase taxonName = taxon.getName();
		if (taxonName == null) {
			return;
		}

		taxonNameText.setText(taxonName.getTitleCache());

		setOkEnabled(true);
	}

	/**
	 * Create contents of the button bar
	 * 
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		setOkEnabled(false);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	public void setOkEnabled(boolean enabled) {
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
	}

	private void initButtons() {
		for (Button button : buttons) {
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(final SelectionEvent e) {
					selectButton((Button) e.getSource());
				}
			});
		}
	}

	/**
	 * Since buttons are not in the same layout group, the others are not
	 * automatically deselected when one is selected. This method takes care of
	 * that.
	 * 
	 * @param button
	 */
	private void selectButton(Button selectedButton) {
		for (Button button : buttons) {
			if (button == selectedButton)
				button.setSelection(true);
			else
				button.setSelection(false);
		}
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(700, 500);
	}

	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setImage(TaxEditorPlugin.getDefault().getImage(
				ITaxEditorConstants.EDIT_ICON));
		newShell.setText(shellText);
	}

	class SelectButtonFocusListener implements FocusListener {

		/**
		 * The button whose group has gained focus.
		 */
		Button button;

		SelectButtonFocusListener(Button button) {
			this.button = button;
		}

		public void focusGained(FocusEvent e) {
			selectButton(button);
		}

		public void focusLost(FocusEvent e) {
		}

	}

	class SelectTaxonDoubleClickListener implements IDoubleClickListener {

		public void doubleClick(DoubleClickEvent event) {
			Taxon taxon = null;
			try {
				taxon = (Taxon) ((StructuredSelection) event.getSelection())
						.getFirstElement();
			} catch (Exception e) {
				e.printStackTrace();
				taxon = null;
			}
			if (taxon.equals(SelectTaxonDialog.this.taxon)) {
				return;
			}

			setSelectedTaxon(taxon);
		}
	}
}
