/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.ui.description.AddDescriptionElementCompositeAction;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.EditorGroupedComposite;
import eu.etaxonomy.taxeditor.editor.ParseListener;

/**
 * Composite for editing a description's label / Title Cache information.
 * 
 * @author p.ciardelli
 * @created 09.09.2008
 * @version 1.0
 */
public class DescriptionLabelComposite extends EditorGroupedComposite {
	private static final Logger logger = Logger
			.getLogger(DescriptionLabelComposite.class);

	/**
	 * ************ FONTS ************
	 */
	public static final Font DESCRIPTION_LABEL_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.ACCEPTED_TAXON_FONT);
	
	/**
	 * ************ ICONS ************
	 */
	public static final Image DESCRIPTION_LABEL_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);	
	
	private static final String EMPTY_NAME_PROMPT = "Click to add description label";
	
	public DescriptionLabelComposite(Composite parent, IManagedForm managedForm, TaxonDescription taxonDescription) {
		super(parent, managedForm);
		
		createLabelViewer();
		createBorderSupport();
		createLineWrapSupport();

		setData(taxonDescription);
		transform(null);

		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		setFocus();
	}

	@Override
	public void transform(String transformation) {
		setDraggable(false);
		setFont(DESCRIPTION_LABEL_FONT);
		setIcon(DESCRIPTION_LABEL_ICON);
		
		createMenu();
	}	
	
	public void setData(Object data) {
		super.setData(data);
		
		Assert.isTrue((data instanceof TaxonDescription), 
				"DescriptionLabelComposite's data field must contain a TaxonDescription object");
		
		String label = ((TaxonDescription) data).getTitleCache();
		getTextViewer().getTextWidget().setText(label);
	}
	
	private void createMenu() {
		
		final Composite parentComposite = this.getParent();
		
		MenuManager featuresMenu = new MenuManager("Add description element");
		featuresMenu.setRemoveAllWhenShown(true);
		featuresMenu.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				for (Feature feature : UiUtil.getPreferredFeatures()) {
					Action action = new AddDescriptionElementCompositeAction(
							parentComposite, managedForm, feature);
					manager.add(action);
				}
			}
		});
		
		// Important: add an empty action. Otherwise, when the main menu sees that
		//	there are no actions in the submenu, it will not be shown
		featuresMenu.add(new Action() {});

		ContextMenu contextMenu = createContextMenu();
		contextMenu.addSubmenu(featuresMenu);
	}

	private void createLabelViewer() {		
		this.textViewer = new SourceViewer(this, null, SWT.WRAP | SWT.MULTI | SWT.RESIZE);
		this.textViewer.setDocument(new Document(""));
		this.textViewer.getTextWidget().setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		this.textViewer.getTextWidget().addModifyListener(new ParseListener() {

			@Override
			public void parse(String text) {
				Assert.isTrue((getData() instanceof TaxonDescription), 
						"DescriptionLabelComposite's data field must contain a TaxonDescription object");
				
				((TaxonDescription) getData()).setTitleCache(text);
				setDirty(true);
			}
			
		});
		this.textViewer.getTextWidget().addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setFocus();
			}			
		});		
	}
}
