/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICellEditorListener;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.springframework.transaction.TransactionStatus;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.UiUtil;
import eu.etaxonomy.taxeditor.actions.TaxonTransfer;
import eu.etaxonomy.taxeditor.actions.cdm.DeleteTaxonAction;
import eu.etaxonomy.taxeditor.actions.cdm.MoveTaxonAction;
import eu.etaxonomy.taxeditor.actions.cdm.SaveTaxonAction;
import eu.etaxonomy.taxeditor.actions.ui.AddQuickNameAction;
import eu.etaxonomy.taxeditor.actions.ui.OpenNewChildNameEditorAction;
import eu.etaxonomy.taxeditor.actions.ui.OpenTaxonEditorAction;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.name.IterableSynonymyList;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.model.TaxonomicTreeContentProvider;

/**
 * Taxon tree viewer which responds to events within individual taxa.
 * 
 * Good overview of TreeViewer:
 * http://www.eclipse.org/articles/Article-TreeViewer/TreeViewerArticle.htm
 * 
 * @author p.ciardelli
 * @created 06.05.2008
 * @version 1.0
 */
public class TaxonomicTreeViewer extends TreeViewer {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeViewer.class);

	private Tree tree;
	private ContextMenu contextMenu;

	private boolean quickAddMode;

	private boolean initialized = false;

	/**
	 * Creates a lazy-loading taxonomic tree that is sorted by TitleCache, and
	 * which listens for name changes and new taxa
	 * 
	 * @param parent
	 */
	public TaxonomicTreeViewer(Composite parent) {

		// SW.VIRTUAL causes nodes to be loaded on-demand, improving performance
		super(parent, SWT.VIRTUAL);

		tree = this.getTree();
		tree.setLinesVisible(false);
		tree.setHeaderVisible(false);

		createContent();
		createToolTips();

		initialized = true;
	}

	public boolean isInitialized() {
		return initialized;
	}

	/**
	 * Tool tips are used to show warnings; there is no tooltip mechanism for
	 * individual tree items out-of-the-box, so tipListener below builds one
	 */
	private void createToolTips() {
		Tree tree = this.getTree();

		// Disable native tooltip
		tree.setToolTipText("");

		tree.addListener(SWT.Dispose, tipListener);
		tree.addListener(SWT.KeyDown, tipListener);
		tree.addListener(SWT.MouseMove, tipListener);
		tree.addListener(SWT.MouseHover, tipListener);
	}

	void createDragAndDrop() {
		final Tree tree = this.getTree();
		Transfer[] types = new Transfer[] { TaxonTransfer.getInstance() };
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;

		final DragSource source = new DragSource(tree, operations);
		source.setTransfer(types);
		final TreeItem[] dragSourceItem = new TreeItem[1];
		source.addDragListener(new DragSourceAdapter() {
			public void dragStart(DragSourceEvent event) {
				TreeItem[] selection = tree.getSelection();
				if (selection.length > 0) {
					event.doit = true;
					dragSourceItem[0] = selection[0];
				} else {
					event.doit = false;
				}
			}

			public void dragSetData(DragSourceEvent event) {
				Taxon taxon = (Taxon) dragSourceItem[0].getData();
				TaxonTransfer.getInstance().setTaxon(taxon);
			}
		});

		DropTarget target = new DropTarget(tree, operations);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {

				// Only drop ONTO another TreeItem OR outside the tree area.
				// Dropping between TreeItems makes no sense because tree is
				// sorted.
				Taxon taxon = (Taxon) dragSourceItem[0].getData();
				Taxon parentTaxon = null;

				// If event.item == null, user has dragged outside of the tree
				if (event.item == null) {
					parentTaxon = null;
				} else {
					parentTaxon = (Taxon) event.item.getData();
					
					// Make sure parentTaxon is not a child
					if (CdmUtil.isTaxonChildOfTaxon(parentTaxon, taxon)) {
						
						MessageDialog.openError(UiUtil.getShell(), "Can't move taxon.",
								"'" + CdmUtil.getDisplayName(taxon) + "' sits above " +
								"'" + CdmUtil.getDisplayName(parentTaxon) + "' " +
								"in the taxonomic hierarchy.");
						
						return;
					}
				}

				new MoveTaxonAction(taxon, parentTaxon).run();
			}
		});
	}
	
	Taxon quickAddTaxon;

	private void removeQuicklyAddedTaxon() {
		if (quickAddTaxon != null) {
			new DeleteTaxonAction(quickAddTaxon).run();
			// if above doesn't work, do remove session taxon but also w remove children
//			TaxEditorPlugin.getDefault().removeSessionTaxon(quickAddTaxon);
			quickAddTaxon = null;
		}
	}

	/**
	 * Taxa with no NameCache - i.e. added with quick name - can be edited in
	 * place in the tax. tree
	 */
	public void createQuickAdd() {

		final Tree tree = this.getTree();
		final TextCellEditor taxTreeNodeEditor = new TextCellEditor(tree);
		taxTreeNodeEditor.addListener(new ICellEditorListener() {
			public void applyEditorValue() {
				Object value = taxTreeNodeEditor.getValue();
				if (value instanceof String) {
					String trimmedValue = ((String) value).trim();

					if (trimmedValue.length() == 0) {
						removeQuicklyAddedTaxon();
					} else {

						NonViralName name = (NonViralName) quickAddTaxon.getName();
//						CdmUtil.parseFullName(name, trimmedValue, null, true);
						CdmUtil.parseFullReference(name, trimmedValue, null, true);
						
//						TaxonNameBase name = CdmUtil.parseFullReference(
//								trimmedValue, null, null);
//						name.addTaxonBase(quickAddTaxon);

						new SaveTaxonAction(quickAddTaxon).run();
						quickAddTaxon = null;
					}
				}
			}

			public void cancelEditor() {
				removeQuicklyAddedTaxon();
			}

			public void editorValueChanged(boolean oldValidState,
					boolean newValidState) {
			}
		});

		this.setCellEditors(new CellEditor[] { taxTreeNodeEditor });
		this.setColumnProperties(new String[] { "col1" });
		this.setCellModifier(new ICellModifier() {
			public boolean canModify(Object element, String property) {

				if (element instanceof Taxon) {
					Taxon taxon = (Taxon) element;

					// If name element has not been initialized,
					// this is a taxon added with QuickAdd					
//					if (taxon.getName() == null) {
//						quickAddTaxon = taxon;
//						return true;
//					}
					
					// BUG Quickly added taxon now retrieved via the quick add action's
					// firePropertyChange - using NULL name to signal this is the
					// quick name taxon was causing odd hibernate errors when quickly
					// added taxon was created and deleted in the same session 
					if (taxon.equals(quickAddTaxon)) {
						return true;
					}
					
				}
				quickAddTaxon = null;
				return false;
			}

			public Object getValue(Object element, String property) {
				// If this node is editable, TitleCache is by definition empty
				return "";
			}

			public void modify(Object element, String property, Object value) {
			}
		});
	}

	/**
	 * On double click, open name editor
	 */
	public void createDoubleClickListener() {
		this.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				Taxon taxon = null;
				if (event.getSelection() instanceof StructuredSelection) {
					Object element = ((StructuredSelection) event
							.getSelection()).getFirstElement();
					if (element instanceof Taxon) {
						new OpenTaxonEditorAction((Taxon) element).run();
					}
				}
			}
		});
	}

	/**
	 * Set up content providers and viewer input
	 */
	private void createContent() {
		
		// Yet another custom content provider ...
		IContentProvider viewerContentProviderList = new TaxonomicTreeContentProvider();
		this.setContentProvider(viewerContentProviderList);

		// Label provider that listens for changes to name cache
		IObservableSet observableTaxonSet = TaxEditorPlugin.getDefault()
				.getObservableSessionTaxa();
		this.setLabelProvider(new TaxonomicTreeLabelProvider(
						observableTaxonSet));

		// TaxonTreeList added to every time a node is opened with its
		// children, or when a new taxon is added
		this.setInput(TaxEditorPlugin.getDefault().getSessionRootTaxa());

		// Sort according to "getColumnText" above, i.e. by NameCache
		this.setComparator(new ViewerComparator());
	}

	/**
	 * If taxon node in the tree is hidden, open and select it
	 * 
	 * @param taxon
	 */
	public void revealTaxon(Taxon taxon) {
		this.setSelection(new StructuredSelection(taxon), true);
		this.reveal(taxon);
	}

	// Implement a "fake" tooltip
	final Listener labelListener = new Listener() {
		public void handleEvent(Event event) {
			Label label = (Label) event.widget;
			Shell shell = label.getShell();
			switch (event.type) {
			case SWT.MouseDown:
				Event e = new Event();
				e.item = (TreeItem) label.getData("_TREEITEM");
				// Assuming table is single select, set the selection as if
				// the mouse down event went through to the table
				// Tree tree = TaxonomicTreeViewer.this.getTree();
				Tree tree = getTree();
				tree.setSelection(new TreeItem[] { (TreeItem) e.item });
				tree.notifyListeners(SWT.Selection, e);
				shell.dispose();
				tree.setFocus();
				break;
			case SWT.MouseExit:
				shell.dispose();
				break;
			}
		}
	};

	Listener tipListener = new Listener() {
		Shell tip = null;
		Label label = null;

		public void handleEvent(Event event) {
			switch (event.type) {
			case SWT.Dispose:
			case SWT.KeyDown:
			case SWT.MouseMove: {
				if (tip == null)
					break;
				tip.dispose();
				tip = null;
				label = null;
				break;
			}
			case SWT.MouseHover: {
				
				// TODO make disappear on ESC
				
				// Item item = TaxonomicTreeViewer.this.getItemAt(new
				// Point(event.x, event.y));
				Item item = getItemAt(new Point(event.x, event.y));
				if (item != null) {
					if (tip != null && !tip.isDisposed())
						tip.dispose();
					tip = new Shell(Display.getCurrent(), SWT.ON_TOP
							| SWT.NO_FOCUS | SWT.TOOL);
					tip.setBackground(Display.getCurrent().getSystemColor(
							SWT.COLOR_INFO_BACKGROUND));
					FillLayout layout = new FillLayout();
					layout.marginWidth = 2;
					tip.setLayout(layout);
					label = new Label(tip, SWT.NONE);
					label.setForeground(Display.getCurrent().getSystemColor(
							SWT.COLOR_INFO_FOREGROUND));
					label.setBackground(Display.getCurrent().getSystemColor(
							SWT.COLOR_INFO_BACKGROUND));
					label.setData("_TABLEITEM", item);
					label.setText(item.getText());
					if (item.getData() instanceof Taxon) {
						Taxon taxon = (Taxon) item.getData();
						IterableSynonymyList synonymyList = new IterableSynonymyList(
								taxon);
						String synonymyListDisplay = CdmUtil
								.getDisplayName(taxon);
						;
						for (TaxonBase synonymOrMisName : synonymyList) {
							TaxonNameBase name = synonymOrMisName.getName();
							if (name != null) {
								synonymyListDisplay += "\n      "
										+ CdmUtil.getDisplayName(name);
							}
						}
						label.setText(synonymyListDisplay);
					}
					// label.setText ("");
					label.addListener(SWT.MouseExit, labelListener);
					label.addListener(SWT.MouseDown, labelListener);
					Point size = tip.computeSize(SWT.DEFAULT, SWT.DEFAULT);
					Rectangle rect = ((TreeItem) item).getBounds(0);
					// Point pt = TaxonomicTreeViewer.this.getTree().toDisplay
					// (rect.x, rect.y);
					Point pt = getTree().toDisplay(rect.x, rect.y);
					tip.setBounds(pt.x, pt.y, size.x, size.y);
					tip.setVisible(true);
				}
			}
			}
		}
	};

	public void createMenu() {
		final MenuManager menuManager = new MenuManager();
		menuManager.setRemoveAllWhenShown(true);
		final Menu menu = menuManager.createContextMenu(tree);
		tree.setMenu(menu);

		menuManager.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {

				Object selection = ((TreeSelection) getSelection())
						.getFirstElement();
				if (!(selection instanceof Taxon)) {
					return;
				}
				Taxon taxon = (Taxon) selection;

				Action openEditorAction = new OpenTaxonEditorAction(taxon);
				manager.add(openEditorAction);

				Action openNewChildEditorAction = new OpenNewChildNameEditorAction(
						taxon);
				manager.add(openNewChildEditorAction);

				Action openQuickNameAction = new AddQuickNameAction(taxon);
				manager.add(openQuickNameAction);
				openQuickNameAction
						.addPropertyChangeListener(new IPropertyChangeListener() {
							public void propertyChange(PropertyChangeEvent event) {
								if (event.getProperty().equals(
										ITaxEditorConstants.QUICK_NAME_TAXON)
										&& event.getNewValue() instanceof Taxon) {
									quickAddTaxon = (Taxon) event.getNewValue();
								}
							}
						});

				manager.add(new Separator());

				Action deleteTaxonAction = new DeleteTaxonAction(taxon);
				manager.add(deleteTaxonAction);
			}
		});
	}
}