/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.propertysheet.namerelationswizard;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * An all-in-one relation wizard page where the user chooses a related name, a
 * relationship type, and the relation's directionality.
 * 
 * Note: does not yet return relation to wizard.
 * 
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class ChooseRelatedNameWizardPage extends WizardPage {
	private static final Logger logger = Logger
			.getLogger(ChooseRelatedNameWizardPage.class);

	private NameRelationWizardModel relationModel;

	private Text txtNewName;
	private Text txtName;
	private boolean nameSelected = false;

	private Table searchResultsTable;
	private WritableList observableSearchResultsList = new WritableList();
	private NonViralName noResultsDummyName = NonViralName.NewInstance(null);

	private Button btnSearch;

	public ChooseRelatedNameWizardPage(NameRelationWizardModel relationModel) {
		super("");

		this.relationModel = relationModel;

		setTitle("Choose related name for '" + relationModel.getBaseName() + "\".");
		setDescription("You can either choose from search results or create a new name. Use \"*\" for wildcard searching.");
	}

	public void createControl(Composite parent) {
		
		// Create composite for popup dialog	
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);
		setControl(container);

		// Create search text
		final Label lblChooseName = new Label(container, SWT.NONE);
		lblChooseName.setText("Choose name for relation:");
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		// Create input field for search term
		txtName = new Text(container, SWT.BORDER);
		txtName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Listen for user hitting <CR> in input field
		txtName.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				int key = e.keyCode;
				if (key == SWT.CR) {
					populateSearchResults();
				}
			}
		});

		// Create "Search" button
		btnSearch = new Button(container, SWT.NONE);
		btnSearch.setEnabled(false);
		btnSearch.setLayoutData(new GridData());
		btnSearch.setText("Search");
		btnSearch.addSelectionListener(new SelectionAdapter() {
			
			// Populate search results resultsTable after clicking button
			public void widgetSelected(SelectionEvent e) {
				populateSearchResults();
			}
		});

		// Create "Create New Name" button
		final Button btnNewName = new Button(container, SWT.NONE);
		btnNewName.setEnabled(false);
		btnNewName.setText("Create New Name");
		btnNewName.addSelectionListener(new SelectionAdapter() {
			
			// Use input field text to create and select a new name
			public void widgetSelected(SelectionEvent e) {
				setRawNewName(txtName.getText());
				btnNewName.setEnabled(false);
			}
		});
		
		// Create results viewer
		createSearchResultsViewer(container);

		// Enable / disable buttons according to whether there is input text or not
		txtName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (txtName.getText().length() > 0) {
					btnNewName.setEnabled(true);
					btnSearch.setEnabled(true);
				} else {
					btnNewName.setEnabled(false);
					btnSearch.setEnabled(false);
				}
			}
		});

		// Create text field to display selected name
		txtNewName = new Text(container, SWT.BORDER);
		final GridData gd_txtNewName = new GridData(SWT.FILL, SWT.CENTER, true,
				false, 3, 1);
		txtNewName.setLayoutData(gd_txtNewName);
		txtNewName.setFont(TaxEditorPlugin.getDefault().getFont
				(ITaxEditorConstants.CHOOSE_NAME_TEXT_FONT));		
		txtNewName.setBackground(Display.getCurrent().getSystemColor(
				SWT.COLOR_WHITE));
		txtNewName.setEditable(false);
		
		// If editing an existing relation, show the related name
		if (relationModel.getRelatedName() != null) {
			txtNewName.setText(CdmUtil.getDisplayName(relationModel.getRelatedName()));
			nameSelected = true;
		}
	}

	protected void populateSearchResults() {
		String searchText = txtName.getText();
		btnSearch.setEnabled(false);
		Set<TaxonNameBase> results = CdmUtil.getNameByName(searchText);

		observableSearchResultsList.clear();
		if (results.size() == 0) {
			// Populate observable list with dummy name
			observableSearchResultsList.add(noResultsDummyName);
		} else {
			observableSearchResultsList.addAll(results);
		}
		btnSearch.setEnabled(true);
	}

	private void createSearchResultsViewer(Composite parent) {
		// TODO no reason to use observable lists and all that nonsense
		// implement  la propertysheet.reference.ReferenceSearchDialog instead
		TableViewer searchResultsViewer = new TableViewer(parent, SWT.BORDER);

		// Add content provider
		ObservableListContentProvider searchResultsProviderList = new ObservableListContentProvider();
		searchResultsViewer.setContentProvider(searchResultsProviderList);

		// Label provider that listens for changes to name cache
		IObservableMap[] searchResultsLabelProviderMaps = BeansObservables
				.observeMaps(searchResultsProviderList.getKnownElements(),
						TaxonNameBase.class, new String[] { "titleCache" });
		searchResultsViewer.setLabelProvider(new ObservableMapLabelProvider(
				searchResultsLabelProviderMaps));

		// Listens for new taxa opened for editing
		searchResultsViewer.setInput(observableSearchResultsList);

		// Sort alphabetically
		searchResultsViewer.setComparator(new ViewerComparator());
		
		// On double click, open name editor
		searchResultsViewer.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				IStructuredSelection selection = (StructuredSelection) event
						.getSelection();
				Object element = selection.getFirstElement();
				if (element instanceof TaxonNameBase) {
					setNewName((TaxonNameBase) element);
				}
			}
		});

		searchResultsTable = searchResultsViewer.getTable();
		searchResultsTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 3, 1));
		searchResultsTable.addListener(SWT.PaintItem, new Listener() {
			/**
			 * If MenuItem with dummy name is being drawn, put it in italics.
			 * 
			 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
			 */
			public void handleEvent(Event event) {
				Object data = event.item.getData();
				if (data instanceof TaxonNameBase
						&& ((TaxonNameBase) data).equals(noResultsDummyName)) {
					TableItem item = (TableItem) event.item;
					item.setFont(TaxEditorPlugin.getDefault().getFont(
									ITaxEditorConstants.MENU_ITEM_ITALICS_FONT));
					item.setText("Search returned no results.");
				}
			}
		});
	}

	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	public boolean isPageComplete() {
		return (nameSelected);
	}
	
	private void updatePage() {
		getWizard().getContainer().updateButtons();
	}

	private void setNewName(TaxonNameBase name) {
		nameSelected = true;
		relationModel.setRelatedName(name);
		txtNewName.setText(CdmUtil.getDisplayName(name));
		updatePage();
	}

	private void setRawNewName(String rawName) {
		nameSelected = true;
		NomenclaturalCode parentCode = relationModel.getBaseName().getNomenclaturalCode();
		TaxonNameBase newName = CdmUtil.parseFullReference(rawName, parentCode, null);
		relationModel.setRelatedName(newName);
		txtNewName.setText(rawName + " (new)");
		updatePage();
	}
}