/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.propertysheet.namerelationswizard;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * An all-in-one relation wizard page where the user chooses a related name, a
 * relationship type, and the relation\"s directionality.
 * 
 * Note: does not yet return relation to wizard.
 * 
 * @author p.ciardelli
 * @created 04.06.2008
 * @version 1.0
 */
public class ChooseRelationTypeWizardPage extends WizardPage {
	private static final Logger logger = Logger
			.getLogger(ChooseRelationTypeWizardPage.class);

	private List<NameRelationshipType> relationTypes;

	private CCombo combo;
	private Label lblDirection;
	private Button btnFromRelatedToBase;
	private Button btnFromBaseToRelated;

	private boolean typeSelected = false;
	private boolean directionSelected = false;
	
	private SelectionListener radioSelectionListener = new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			directionSelected = true;
			updatePage();
			
			if (e.widget.equals(btnFromRelatedToBase)) {
				relationModel.setFromName(relationModel.getRelatedName());
				relationModel.setToName(relationModel.getBaseName());
			}
			
			if (e.widget.equals(btnFromBaseToRelated)) {
				relationModel.setFromName(relationModel.getBaseName());
				relationModel.setToName(relationModel.getRelatedName());
			}
		}
	};

	private NameRelationWizardModel relationModel; 

	public ChooseRelationTypeWizardPage(final NameRelationWizardModel relationModel) {
		super("");
		setTitle("Choose relationship type and direction.");		
		
		this.relationModel = relationModel;
		
		setDescription();
		
		relationModel.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (evt.getPropertyName().equals(NameRelationWizardModel.RELATEDNAME)) {
					setDescription();
					setDirectionLabels();
				}
				if (evt.getPropertyName().equals(NameRelationWizardModel.RELATIONTYPE)) {
					if (relationModel.getType() != null) {
						typeSelected = true;
						enableDirection();
					} else {
						typeSelected = false;
						disableDirection();
					}
					updatePage();
				}				
			}
		});
	}
		
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new GridLayout());

		setControl(container);

		relationTypes = new ArrayList<NameRelationshipType>();

		final Label lblType = new Label(container, SWT.NONE);
		lblType.setText("Choose relationship type:");

		combo = new CCombo(container, SWT.BORDER);
		combo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		for (NameRelationshipType relationType : TaxEditorPlugin.getDefault()
				.getSortedNameRelationshipTypes()) {
			relationTypes.add(relationType);
			
			String label = relationType.getLabel();
			if (relationModel.isZoological() && 
					relationType.equals(NameRelationshipType.BASIONYM())) {
				label = "original combination";
			} 
			combo.add(label);
		}
		combo.setVisibleItemCount(relationTypes.size());

		combo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				int typeIndex = combo.getSelectionIndex();
				relationModel.setType(relationTypes.get(typeIndex));
			}
		});

		final Group group = new Group(container, SWT.NONE);
		group.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		final GridLayout gridLayout = new GridLayout();
		group.setLayout(gridLayout);

		lblDirection = new Label(group, SWT.NONE);
		final GridData gd_lblDirection = new GridData(SWT.FILL, SWT.CENTER, true, false);
		gd_lblDirection.heightHint = 32;
		gd_lblDirection.minimumHeight = 40;
		lblDirection
				.setLayoutData(gd_lblDirection);
		lblDirection.setEnabled(false);
		lblDirection.setText("Choose relationship direction:");

		btnFromRelatedToBase = new Button(group, SWT.RADIO);
		btnFromRelatedToBase
				.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		btnFromRelatedToBase.setVisible(false);
		btnFromRelatedToBase.addSelectionListener(radioSelectionListener);

		btnFromBaseToRelated = new Button(group, SWT.RADIO);
		btnFromBaseToRelated
				.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		btnFromBaseToRelated.setVisible(false);
		btnFromBaseToRelated.addSelectionListener(radioSelectionListener);
		new Label(group, SWT.NONE);

	}

	private void setDescription() {
		setDescription("Choose relationship type and direction for the name relation between \""
				+ CdmUtil.getDisplayName(relationModel.getBaseName()) + "\" and \"" 
				+ CdmUtil.getDisplayName(relationModel.getRelatedName()) + "\".");		
	}
	
	private void setDirectionLabels() {
		String strDirection1 = "\"" + CdmUtil.getDisplayName(relationModel.getRelatedName()) + "\" "
				+ "is \"" + combo.getText() + "\" of " + "\""
				+ CdmUtil.getDisplayName(relationModel.getBaseName()) + "\"";
		String strDirection2 = "\"" + CdmUtil.getDisplayName(relationModel.getBaseName()) + "\" "
				+ "is \"" + combo.getText() + "\" of " + "\""
				+ CdmUtil.getDisplayName(relationModel.getRelatedName()) + "\"";
		
		btnFromRelatedToBase.setText(strDirection1);
		btnFromBaseToRelated.setText(strDirection2);		
	}
	
	@Override
	public boolean canFlipToNextPage() {
		// Last page
		return false;
	}

    public boolean isPageComplete() {
        return (typeSelected && directionSelected);
    }

	private void updatePage() {
		if (typeSelected) {
			if (!directionSelected) {
				enableDirection();
			}
		} else {
			disableDirection();
		}
		getWizard().getContainer().updateButtons();
	}
	
	private void enableDirection() {
		setDirectionLabels();

		lblDirection.setEnabled(true);
		btnFromRelatedToBase.setEnabled(true);
		btnFromBaseToRelated.setEnabled(true);

		btnFromRelatedToBase.setVisible(true);
		btnFromBaseToRelated.setVisible(true);
	}

	private void disableDirection() {
		lblDirection.setEnabled(false);
		btnFromRelatedToBase.setEnabled(false);
		btnFromBaseToRelated.setEnabled(false);

		btnFromRelatedToBase.setSelection(false);
		btnFromBaseToRelated.setSelection(false);
	}
}